<?php

namespace Drupal\proc\Element;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Render\Element\FormElement;
use Drupal\Core\Url;
use Drupal\proc\Entity\Proc;

/**
 * Provides a form element for encryption and decryption.
 *
 * @FormElement("procfield")
 */
class ProcField extends FormElement {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $class = get_class($this);
    return [
      '#input' => TRUE,
      '#process' => [
        [$class, 'processProcField'],
      ],
      '#theme_wrappers' => ['form_element'],
    ];
  }

  /**
   * Processes a proc field form element.
   */
  public static function processProcField(&$element, FormStateInterface $form_state, &$complete_form) {
    $config = \Drupal::config('proc.settings');

    $proc_field_name = $element['proc-settings']['proc_field_name'];

    $field_input_mode = $element['proc-settings']['field_input_mode'];
    $to_recipients_field_name = $element['proc-settings']['to_recipients_field_name'];
    $carbon_copy_recipients_field_name = $element['proc-settings']['carbon_copy_recipients_field_name'];
    $preset_recipients_csv = $element['proc-settings']['preset_recipients_csv'];
    $fetcher_endpoint = $element['proc-settings']['fetcher_endpoint'];
    $fetcher_endpoint_filter = $element['proc-settings']['fetcher_endpoint_filter'];
    $fetcher_endpoint_filter_type = $element['proc-settings']['fetcher_endpoint_filter_type'];
    $cache_password = $element['proc-settings']['cache_password'];
    $trigger_format = $element['proc-settings']['trigger_format'];
    $submit_element_id = $element['proc-settings']['submit_element_id'];
    $trigger_fields = $element['proc-settings']['trigger_fields'];
    $proc_field_decryption_mode = $element['proc-settings']['proc_field_decryption_mode'];

    // JavaScript variables.
    $direct_fetcher_js_prefix = <<<JS

      let to_recipients_collection = [];
      let cc_recipients_collection = [];
      let pre_set_recipients_csv = "{$preset_recipients_csv}";
      let fetcher_endpoint = "{$fetcher_endpoint}";
      let fetcher_endpoint_filter = "{$fetcher_endpoint_filter}";
      let fetcher_endpoint_filter_type = "{$fetcher_endpoint_filter_type}";
      let recipients = [];
    JS;

    $direct_fetcher_to = !empty($to_recipients_field_name) ? <<<JS

      to_recipients_collection = Object.values(document.querySelectorAll("input[name^='{$to_recipients_field_name}']"));
    JS : '';

    $direct_fetcher_cc = !empty($carbon_copy_recipients_field_name) ? <<<JS

      cc_recipients_collection = Object.values(document.querySelectorAll("input[name^='{$carbon_copy_recipients_field_name}']"));
    JS : '';

    $fetcher_value = <<<JS
      let proc_path = proc_path_prefix_add + "/" + ids_csv + proc_path_suffix;
      let ajaxSettings = {
        url: proc_path,
        dialogType: "dialog",
        dialog: { width: 400, classes: { "ui-dialog": "class-{$proc_field_name}" } },
      };
      let myAjaxObject = Drupal.ajax(ajaxSettings);
      myAjaxObject.execute();
    JS;

    if ($fetcher_endpoint) {
      $fetcher_value = <<<JS
        // Fetcher endpoint CSV:
        let fetcher_endpoint_csv = "";
        let endpoint_response;
        let dynamic_endpoint_filter;
        let dynamic_endpoint_filter_prefix;

        if (fetcher_endpoint_filter != "") {
          // Turn fetcher_endpoint_filter CSV into an array:
          const filter_element_identifiers = fetcher_endpoint_filter.split(",");
          let filter_element_values = [];
          const identifier_type_string = fetcher_endpoint_filter_type == '0' ? 'id' : 'name';
          filter_element_identifiers.map((fieldIdentifier, fieldIndex) => {
            // Get the value of the field:
            let field_value = document.querySelector('[' + identifier_type_string + '^="' + fieldIdentifier + '"]').value;
            filter_element_values.push(fieldIdentifier + "=" + field_value);
          });
          dynamic_endpoint_filter = filter_element_values.join("&");
          dynamic_endpoint_filter_prefix = "?";
        }
        if (fetcher_endpoint != "") {
          if (dynamic_endpoint_filter) {
            fetcher_endpoint = fetcher_endpoint + dynamic_endpoint_filter_prefix + dynamic_endpoint_filter;
          }
          console.info("Fetching recipients from endpoint: " + fetcher_endpoint);

          endpoint_response = fetch(fetcher_endpoint, {
              method: "get"
          }).then(async function(response) {
            return response.json();
          }).catch(async function(err) {
            console.info("Error.");
          });
        }
        let endpoint_recipients_csv = endpoint_response.then(async function(result) {
          return result.map(item => item.id).join();
        });
        endpoint_recipients_csv.then(function(result) {
          if (result != "") {
            var separator = "";
            if (ids_csv != "") {
              separator = ",";
            }
            ids_csv = ids_csv + separator + result;
            proc_path = proc_path_prefix_add + "/" + ids_csv + proc_path_suffix;

            ajaxSettings = {
              url: proc_path,
              dialogType: "dialog",
              dialog: { width: 400, classes: { "ui-dialog": "class-{$proc_field_name}" } },
            };

            let myAjaxObject = Drupal.ajax(ajaxSettings);
            myAjaxObject.execute();
            return false;
          }
        });
      JS;
    }

    $direct_fetcher_js = <<<JS

      jQuery(this).attr("href", "#");

      // Check if this is an autocomplete deluxe field:
      if (to_recipients_collection.length) {
        if (to_recipients_collection[0].value == "" && to_recipients_collection[1].value.includes('""""')) {
          // This is an autocomplete deluxe field. Get values from the second element.
          let autocomplete_deluxe_recipients_string = to_recipients_collection[1].value + cc_recipients_collection[1].value;
          const regex = /\((\d+)\)/g;
          const matches = autocomplete_deluxe_recipients_string.match(regex);
          recipients = matches.map((match) => parseInt(match.match(/\d+/)));

        }
        else {
          // Suppose this is a regular autocomplete field:
          let recipients_length = to_recipients_collection.length + cc_recipients_collection.length;
          if (recipients_length > 0) {
            let recipients_collection = to_recipients_collection.concat(cc_recipients_collection);
            recipients_collection.forEach(function (recipient, index) {
              if (index < recipients_length) {
                if (recipient.value.match(/\(\d+\)/)) {
                  let id_parenthesis = recipient.value.match(/\(\d+\)/)[0];
                  recipients.push(id_parenthesis.substring(1, id_parenthesis.length - 1));
                }
              }
            });
          }
        }
      }

      let proc_path_prefix_add = window.location.origin + drupalSettings.path.baseUrl + "proc/add";
      let ids_csv = recipients.join();

      // Concatenate predefined recipients:
      if (pre_set_recipients_csv != "") {
        ids_csv = ids_csv + "," + pre_set_recipients_csv;
      }

      let proc_field_name = "{$proc_field_name}";
      let parent_id = document.querySelector("input[name^='" + proc_field_name + "']").id;

      let field_input_mode = {$field_input_mode};

      let proc_path_suffix = "?proc_standalone_mode=FALSE&proc_parent_id=" + parent_id + "&proc_field_name={$proc_field_name}&field-input-mode=" + field_input_mode;

      {$fetcher_value}
      return false;
    JS;

    $direct_fetcher_js = $direct_fetcher_js_prefix . $direct_fetcher_to . $direct_fetcher_cc . $direct_fetcher_js;

    // Create the URL for the Encrypt button.
    $url = Url::fromUserInput('#', [
      'attributes' => [
        'onclick' => $direct_fetcher_js,
        'id' => 'encrypt-' . $proc_field_name,
      ],
    ]);

    // Create the Encrypt button link.
    $encrypt_button_class = [$config->get('proc-encrypt-button-class')];
    if ($trigger_format === '1') {
      $encrypt_button_class = ['hidden'];
    }

    $link = [
      '#title' => $config->get('proc-encrypt-button-label'),
      '#type' => 'link',
      '#url' => $url,
      '#attributes' => [
        'class' => $encrypt_button_class,
        'name' => 'encrypt-' . $proc_field_name,
        'id' => 'encrypt-' . $proc_field_name,
      ],
    ];

    // Create the checkbox for encryption.
    $checkbox = [
      '#type' => 'checkbox',
      '#title' => 'Encrypt',
      '#default_value' => [0 => 0],
      '#id' => 'encrypt-checkbox-' . $proc_field_name,
      '#attributes' => [
        'onclick' => <<<JS

            if (this.checked) {
                document.getElementById('encrypt-{$proc_field_name}').click();
                this.value = 1;
            }
            else {
                document.getElementById('decrypt-{$proc_field_name}').click();
                this.value = 0;
            }
        JS,
        'value' => 0,
      ],
    ];

    $element['target_id']['#attached']['library'][] = 'proc/proc-field';

    if ($submit_element_id) {
      $element['target_id']['#attached']['drupalSettings']['proc']['submit_element_id'] = $submit_element_id;
    }

    $decryption_link = [];

    $decryption_mode = FALSE;
    $proc_id = '';
    // If there is a default value, add also the Decrypt button:
    if ($element['target_id']['#default_value']) {

      $label = $element['target_id']['#default_value']->get('label')->getValue()[0]['value'];

      $decryption_mode = $proc_field_decryption_mode ?? FALSE;

      $proc_id = $element['target_id']['#default_value']->get('id')->getValue()[0]['value'];
      $decryption_url = Url::fromUserInput(
        '/proc/' . $proc_id
      );

      $decryption_link = Link::fromTextAndUrl(t('Decrypt'), $decryption_url);
      $decryption_link = $decryption_link->toRenderable();
      $decryption_link['#attributes']['id'] = 'decrypt-' . $proc_field_name;

      // If decryption has to happen in a dialog:
      if ($decryption_mode == '1') {
        $dialog_decrypt_js = <<<JS
          let proc_path_prefix_add = window.location.origin + drupalSettings.path.baseUrl + "proc/{$proc_id}";

          // let parent_selector = jQuery(this).parent();
          // let parent_id = parent_selector[0].getAttribute("id").slice(0, -13);

          let proc_field_name = "{$proc_field_name}";
          let parent_id = document.querySelector("input[name^='" + proc_field_name + "']").id;

          let field_input_mode = {$field_input_mode};
          let field_decryption_mode = {$decryption_mode};
          const cache_password = {$cache_password};

          let proc_path_suffix = "?proc_standalone_mode=FALSE&cache-password=" + cache_password + "&proc_parent_id=" + parent_id + "&proc_field_name={$proc_field_name}&field-input-mode=" + field_input_mode + "&field-decryption-mode=" + field_decryption_mode + "&trigger-fields={$trigger_fields}";
          let proc_path = proc_path_prefix_add + "/" + proc_path_suffix;

          jQuery(this).attr("href", "#");

          let ajaxSettings = {
            url: proc_path,
            dialogType: "dialog",
            dialog: { width: 400, classes: { "ui-dialog": "class-{$proc_field_name}" } },
          };
          let myAjaxObject = Drupal.ajax(ajaxSettings);
          myAjaxObject.execute();
          return false;
        JS;

        $dialog_decrypt_url = Url::fromUserInput(
        // URL with recipients IDs will be defined
        // by the direct fetcher.
          '#',
          [
            'attributes' => [
              'onclick' => $dialog_decrypt_js,
              'class' => [$config->get('proc-encrypt-button-class')],
              'id' => 'decrypt-' . $proc_field_name,
            ],
          ]
        );
        $decryption_link['#url'] = $dialog_decrypt_url;
      }
    }

    $description_proc_elements = [$link];
    if ($trigger_format === '1') {
      $description_proc_elements = [
        $link,
        $decryption_link,
        $checkbox,
      ];
    }

    $element['proc-ops'] = $description_proc_elements;

    $decryption_link['#attributes']['class'] = 'hidden';

    if (isset($decryption_link['#type']) && $decryption_link['#type'] == 'link') {
      $checkbox['#attributes']['checked'] = 'checked';
      $element['proc-ops'] = [
        $link,
        $decryption_link,
        $checkbox,
      ];
      if ($trigger_format === '0') {
        $decryption_link['#attributes']['class'] = [$config->get('proc-encrypt-button-class')];
        $element['proc-ops'] = [$link, $decryption_link];
      }
    }

    // If this is an encrypted text field:
    if ($field_input_mode == 2) {
      $proc_object = FALSE;
      if ($proc_id) {
        $proc_object = Proc::load($proc_id);
      }
      $element['target_id']['#type'] = 'textfield';
      $element['target_id']['#value'] = $label ?? '';

      // Set the element validate method:
//      $element['#element_validate'] = [
//        [$proc_object, 'validateProcField'],
//      ];

      // Add hidden field for proc_id:
      $element['proc-ops'] = [$element['proc-ops']];
      $element['target_id']['#default_value'] = $proc_object ?? '';
    }

    $element['target_id']['#attributes']['proc'] = 'true';

    return $element;
  }

  /**
   * Validates a procfield element.
   */
  public static function validateProcField(&$element, FormStateInterface $form_state, &$complete_form) {
    return $element;
  }
}
