<?php

namespace Drupal\proc\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldWidget\EntityReferenceAutocompleteWidget;
use Drupal\Core\Form\FormStateInterface;

/**
 * Defines the 'proc_entity_reference_widget' field widget.
 *
 * @FieldWidget(
 *   id = "proc_entity_reference_widget",
 *   module = "proc",
 *   label = @Translation("Proc Entity Reference Field Widget"),
 *   field_types = {
 *     "proc_entity_reference_field"
 *   }
 * )
 */
class ProcEntityReferenceWidget extends EntityReferenceAutocompleteWidget {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
        'proc_field_recipients_fetcher_endpoint' => '',
        'proc_field_recipients_fetcher_endpoint_filter' => '',
        'proc_field_recipients_fetcher_endpoint_filter_type' => '',
        'proc_field_recipients_manual_fetcher' => '',
        'proc_field_recipients_to_field' => '',
        'proc_field_recipients_cc_field' => '',
        'proc_field_mode' => 1,
        'proc_field_input_mode' => 0,
        'proc_field_decryption_mode' => '',
        'proc_allow_password_caching' => '',
        'proc_trigger_format' => '',
        'proc_master_field_submit_id' => '',
        'proc_decryption_trigger_fields' => '',
      ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element[] = parent::settingsForm($form, $form_state);

    $element['proc_field_recipients_to_field'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Machine name of the field containing the user IDs of recipients'),
      '#description' => $this->t('It must be a user reference field visible in the same form as this field.'),
      '#default_value' => $this->getSetting('proc_field_recipients_to_field') ?? '',
    ];

    $element['proc_field_recipients_cc_field'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Machine name of the field containing user IDs of CC recipients'),
      '#description' => $this->t('It must be a user reference field visible in the same form as this field.'),
      '#default_value' => $this->getSetting('proc_field_recipients_cc_field') ?? '',
    ];

    $element['proc_field_recipients_manual_fetcher'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Define the user IDs of recipients in a CSV list'),
      '#description' => $this->t('Example: 1,2,3'),
      '#default_value' => $this->getSetting('proc_field_recipients_manual_fetcher') ?? '',
      '#disabled' => FALSE,
    ];

    $element['proc_field_recipients_fetcher_endpoint'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Endpoint for fetching user IDs of recipients'),
      '#description' => $this->t('Example: https://example.com/api/get/users/param-a/param-b/param-c?param-d=1&#38;param-e=2'),
      '#default_value' => $this->getSetting('proc_field_recipients_fetcher_endpoint') ?? '',
      '#disabled' => FALSE,
    ];

    $element['proc_field_recipients_fetcher_endpoint_filter'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Identifier of form elements to be used as parameters for the endpoint'),
      '#description' => $this->t('Example: edit-field-a,edit-field-b,edit-field-c'),
      '#default_value' => $this->getSetting('proc_field_recipients_fetcher_endpoint_filter') ?? '',
      '#disabled' => FALSE,
    ];

    $element['proc_field_recipients_fetcher_endpoint_filter_type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Type of identifier'),
      '#default_value' => $this->getSetting('proc_field_recipients_fetcher_endpoint_filter_type') ?? 0,
      '#options' => [
        0 => $this->t('ID'),
        1 => $this->t('Name'),
      ],
      '#disabled' => FALSE,
    ];

    $element['proc_field_mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Modes of operation'),
      '#default_value' => $this->getSetting('proc_field_mode') ?? 0,
      '#options' => [
        0 => $this->t('Disabled'),
        1 => $this->t('Only encryption'),
        2 => $this->t('Encryption and signature'),
      ],
      2 => [
        '#disabled' => TRUE,
      ],
    ];

    $element['proc_field_input_mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Modes of input'),
      '#default_value' => $this->getSetting('proc_field_input_mode') ?? '',
      '#options' => [
        0 => $this->t('File'),
        1 => $this->t('Text area'),
        2 => $this->t('Text field'),
      ],
      1 => [
        '#disabled' => TRUE,
      ],
    ];

    $element['proc_field_decryption_mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Modes of decryption'),
      '#default_value' => $this->getSetting('proc_field_decryption_mode') ?? '',
      '#options' => [
        0 => $this->t('New page'),
        1 => $this->t('Dialog'),
      ],
    ];

    $element['proc_allow_password_caching'] = [
      '#type' => 'radios',
      '#title' => $this->t('Password caching'),
      '#default_value' => $this->getSetting('proc_allow_password_caching') ?? '',
      '#options' => [
        0 => $this->t('Disable password caching'),
        1 => $this->t('Show checkbox for allowing password caching'),
        2 => $this->t('Hide the checkbox and apply password caching by default'),
      ],
    ];

    $element['proc_trigger_format'] = [
      '#type' => 'radios',
      '#title' => $this->t('Trigger format'),
      '#default_value' => $this->getSetting('proc_trigger_format') ?? '',
      '#options' => [
        0 => $this->t('Encrypt and Decrypt buttons'),
        1 => $this->t('Toggle checkbox (check for encryption, uncheck for decryption)'),
      ],
    ];

    $element['proc_master_field_submit_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Submit element ID for the master field'),
      '#description' => $this->t('Define the submit element ID of the form where this field is located. Example: edit-submit'),
      '#default_value' => $this->getSetting('proc_master_field_submit_id') ?? '',
      '#disabled' => FALSE,
    ];

    $element['proc_decryption_trigger_fields'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Trigger decryption of another encrypted field'),
      '#default_value' => $this->getSetting('proc_decryption_trigger_fields') ?? '',
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary[] = parent::settingsSummary();

    $operators = $this->getMatchOperatorOptions();
    $summary[] = $this->t('Autocomplete matching: @match_operator', ['@match_operator' => $operators[$this->getSetting('match_operator')]]);
    $size = $this->getSetting('match_limit') ?: $this->t('unlimited');
    $summary[] = $this->t('Autocomplete suggestion list size: @size', ['@size' => $size]);
    $summary[] = $this->t('Textfield size: @size', ['@size' => $this->getSetting('size')]);
    $placeholder = $this->getSetting('placeholder');
    if (!empty($placeholder)) {
      $summary[] = $this->t('Placeholder: @placeholder', ['@placeholder' => $placeholder]);
    }
    else {
      $summary[] = $this->t('No placeholder');
    }
    $proc_recipients_to_field = $this->getSetting('proc_field_recipients_to_field');
    $summary[] = $this->t('Machine name of recipients To: @proc_field_recipients_to_field' , ['@proc_field_recipients_to_field' => $proc_recipients_to_field]);
    $proc_recipients_cc_field = $this->getSetting('proc_field_recipients_cc_field');
    $summary[] = $this->t('Machine name of recipients CC: @proc_field_recipients_cc_field' , ['@proc_field_recipients_cc_field' => $proc_recipients_cc_field]);
    $proc_field_recipients_manual_fetcher = $this->getSetting('proc_field_recipients_manual_fetcher');
    $summary[] = $this->t('Default recipients: @proc_field_recipients_manual_fetcher' , ['@proc_field_recipients_manual_fetcher' => $proc_field_recipients_manual_fetcher]);
    $proc_field_recipients_fetcher_endpoint = $this->getSetting('proc_field_recipients_fetcher_endpoint');
    $summary[] = $this->t('Recipients fetcher endpoint: @proc_field_recipients_fetcher_endpoint' , ['@proc_field_recipients_fetcher_endpoint' => $proc_field_recipients_fetcher_endpoint]);
    $proc_field_recipients_fetcher_endpoint_filter = $this->getSetting('proc_field_recipients_fetcher_endpoint_filter');
    $summary[] = $this->t('Recipients fetcher: @proc_field_recipients_fetcher_endpoint_filter', ['@proc_field_recipients_fetcher_endpoint_filter' => $proc_field_recipients_fetcher_endpoint_filter]);
    $proc_field_recipients_fetcher_endpoint_filter_type = $this->getSetting('proc_field_recipients_fetcher_endpoint_filter_type');
    $summary[] = $this->t('Recipients fetcher type: @proc_field_recipients_fetcher_endpoint_filter_type', ['@proc_field_recipients_fetcher_endpoint_filter_type' => $proc_field_recipients_fetcher_endpoint_filter_type]);
    $proc_field_mode = $this->getSetting('proc_field_mode');
    $summary[] = $this->t('Field mode: @proc_field_mode', ['@proc_field_mode' => $proc_field_mode]);
    $proc_field_input_mode = $this->getSetting('proc_field_input_mode');
    $summary[] = $this->t('Input mode: @proc_field_input_mode', ['@proc_field_input_mode' => $proc_field_input_mode]);
    $proc_field_decryption_mode = $this->getSetting('proc_field_decryption_mode');
    $summary[] = $this->t('Decryption mode: @proc_field_decryption_mode', ['@proc_field_decryption_mode' => $proc_field_decryption_mode]);
    $proc_allow_password_caching = $this->getSetting('proc_allow_password_caching');
    $summary[] = $this->t('Password caching: @proc_allow_password_caching', ['@proc_allow_password_caching' => $proc_allow_password_caching]);
    $proc_trigger_format = $this->getSetting('proc_trigger_format');
    $summary[] = $this->t('Trigger format: @proc_trigger_format', ['@proc_trigger_format' => $proc_trigger_format]);
    $proc_master_field_submit_id = $this->getSetting('proc_master_field_submit_id');
    $summary[] = $this->t('Submit element ID: @proc_master_field_submit_id', ['@proc_master_field_submit_id' => $proc_master_field_submit_id]);
    $proc_decryption_trigger_fields = $this->getSetting('proc_decryption_trigger_fields');
    $summary[] = $this->t('Decryption trigger fields: @proc_decryption_trigger_fields', ['@proc_decryption_trigger_fields' => $proc_decryption_trigger_fields]);

    return $summary;
  }


  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    // Call the parent formElement method to get the base element.
    $element = parent::formElement($items, $delta, $element, $form, $form_state);

    // Extract widget settings into local variables.
    // @TODO: Move these to class properties.
    $element['proc-settings']['field_input_mode'] = $this->getSetting('proc_field_input_mode') ?? '';
    $element['proc-settings']['to_recipients_field_name'] = $this->getSetting('proc_field_recipients_to_field') ?? '';
    $element['proc-settings']['carbon_copy_recipients_field_name'] = $this->getSetting('proc_field_recipients_cc_field') ?? '';
    $element['proc-settings']['preset_recipients_csv'] = $this->getSetting('proc_field_recipients_manual_fetcher') ?? '';
    $element['proc-settings']['fetcher_endpoint'] = $this->getSetting('proc_field_recipients_fetcher_endpoint') ?? '';
    $element['proc-settings']['fetcher_endpoint_filter'] = $this->getSetting('proc_field_recipients_fetcher_endpoint_filter') ?? '';
    $element['proc-settings']['fetcher_endpoint_filter_type'] = $this->getSetting('proc_field_recipients_fetcher_endpoint_filter_type') ?? '';
    $element['proc-settings']['cache_password'] = $this->getSetting('proc_allow_password_caching') ?? '0';
    $element['proc-settings']['trigger_format'] = $this->getSetting('proc_trigger_format') ?? '0';
    $element['proc-settings']['submit_element_id'] = $this->getSetting('proc_master_field_submit_id') ?? '';
    $element['proc-settings']['trigger_fields'] = $this->getSetting('proc_decryption_trigger_fields') ?? '0';
    $element['proc-settings']['proc_field_decryption_mode'] = $this->getSetting('proc_field_decryption_mode') ?? '';
    $element['proc-settings']['proc_field_name'] = $items->getName();
    $element['proc-settings']['proc_field_mode'] = $this->getSetting('proc_field_mode') ?? 0;

    if ($element['proc-settings']['proc_field_mode'] !== "0") {
      $element['#type'] = 'procfield';
    }

    return $element;
  }
}
